@@ -48,7 +48,8 @@
     // view is an NSBox. we need its contents.
     view = [[(NSBox *)view contentView] retain];
 
-    [matrix registerForDraggedTypes:[NSArray arrayWithObject:NSColorPboardType]];
+    [matrix registerForDraggedTypes:
+                                 [NSArray arrayWithObject:NSColorPboardType]];
     [matrix setDelegate:self];
 //    [matrix setCellClass:[ColoredSwitchButtonCell class]];
 
@@ -65,6 +66,7 @@
 
 - (void)dealloc
 {
+    [entityTypePopUp removeAllItems];
     [filters release];
     [hiddenEntityTypes release];
     [view release];
@@ -273,13 +275,18 @@
 
 - (void)setConfiguration:(id)config
 {
+    NSDictionary *configuration;
     NSMutableDictionary *hiddenContainers;
     NSEnumerator *keyEnum;
     id key;
     NSArray *value;
     NSArray *hiddenTypes;
 
-    hiddenContainers = [[config objectForKey:@"HiddenContainers"] unifyStrings];
+    NSParameterAssert([config isKindOfClass:[NSDictionary class]]);
+    configuration = config;
+
+    hiddenContainers = [[configuration objectForKey:@"HiddenContainers"]
+                                                                unifyStrings];
 
     keyEnum = [hiddenContainers keyEnumerator];
     while ((key = [keyEnum nextObject]) != nil) {
@@ -287,7 +294,8 @@
 	[filters setObject:[NSMutableSet setWithArray:value] forKey:key];
     }
 
-    hiddenTypes = [[config objectForKey:@"HiddenContainerTypes"] unifyStrings];
+    hiddenTypes = [[configuration objectForKey:@"HiddenContainerTypes"]
+                                                               unifyStrings];
     Assign(hiddenEntityTypes, [NSMutableSet setWithArray:hiddenTypes]);
     [self hierarchyChanged];
     [super dataChangedForEntityType:nil];
@@ -359,7 +367,6 @@
     id entityName;
     int i = 0;
     ColoredSwitchButtonCell *cell;
-    NSMutableAttributedString *title;
 
     if (entityType == nil) {
         return;
@@ -379,6 +386,7 @@
         [cell setRepresentedObject:entityName];
 #if 0
 #ifndef xxGNUSTEP
+        NSMutableAttributedString *title;
         title = [[[NSMutableAttributedString alloc] initWithString:[entityName description]] autorelease];
 #if defined(__APPLE__)// || defined(GNUSTEP)
         [title replaceCharactersInRange:NSMakeRange(0,0) withString:@"# "];
@@ -475,10 +483,14 @@
     return filteredTypes;
 }
 
-- (BOOL)isHiddenContainer:(PajeContainer *)container
-                   filter:(NSSet *)filter
+- (id)filterHiddenContainer:(PajeContainer *)container
+                     filter:(NSSet *)filter
 {
-    return [filter containsObject:[self nameForEntity:container]];
+    if ([filter containsObject:[self nameForEntity:container]]) {
+        return nil;
+    } else {
+        return container;
+    }
 }
 
 - (NSEnumerator *)enumeratorOfContainersTyped:(PajeContainerType *)entityType
@@ -488,7 +500,7 @@
     NSMutableSet *filter;
 
     if ([self isHiddenEntityType:entityType]) {
-        NSLog(@"enumerating hidden type %@", entityType);
+        NSWarnLog(@"enumerating hidden type %@", entityType);
         return nil;
     }
 
@@ -497,7 +509,11 @@
 
     filter = [self filterForEntityType:entityType];
     if (filter != nil) {
-        return [[[FilteredEnumerator alloc] initWithEnumerator:origEnum filter:self selector:@selector(isHiddenContainer:filter:) context:filter] autorelease];
+        return [[[FilteredEnumerator alloc]
+                    initWithEnumerator:origEnum
+                                filter:self
+                              selector:@selector(filterHiddenContainer:filter:)
+                               context:filter] autorelease];
     } else {
         return origEnum;
     }
