@@ -1,5 +1,5 @@
 /*
-    Copyright (c) 1998, 1999, 2000, 2001, 2003, 2004 Benhur Stein
+    Copyright (c) 1998--2005 Benhur Stein
     
     This file is part of Paj.
 
@@ -71,6 +71,7 @@
 
 - (void)dealloc
 {
+    [entityTypePopUp removeAllItems];
     [filters release];
     [hiddenEntityTypes release];
     [view release];
@@ -224,14 +225,15 @@
     PajeEntityType *entityType = [self selectedEntityType];
     NSMutableSet *filter = [filters objectForKey:entityType];
 
-    if (filter) {
+    if (filter != nil) {
         [filter removeAllObjects];
         [self synchronizeMatrix];
         [self registerDefaultFilter:filter forEntityType:entityType];
         [super dataChangedForEntityType:entityType];
     } else {
         [NSException raise:@"EntityTypeSelector: Internal Inconsistency"
-                    format:@"%@ sent by unknown view %@", NSStringFromSelector(_cmd), sender];
+                    format:@"%@ sent by unknown view %@",
+                             NSStringFromSelector(_cmd), sender];
     }
 }
 
@@ -245,14 +247,16 @@
     PajeEntityType *entityType = [self selectedEntityType];
     NSMutableSet *filter = [filters objectForKey:entityType];
 
-    if (filter) {
-        [filter addObjectsFromArray:[self unfilteredObjectsForEntityType:entityType]];
+    if (filter != nil) {
+        [filter addObjectsFromArray:
+                              [self unfilteredObjectsForEntityType:entityType]];
         [self synchronizeMatrix];
         [self registerDefaultFilter:filter forEntityType:entityType];
         [super dataChangedForEntityType:entityType];
     } else {
         [NSException raise:@"EntityTypeSelector: Internal Inconsistency"
-                    format:@"%@ sent by unknown view %@", NSStringFromSelector(_cmd), sender];
+                    format:@"%@ sent by unknown view %@",
+                            NSStringFromSelector(_cmd), sender];
     }
 }
 
@@ -328,7 +332,6 @@
     id entityName;
     int i = 0;
     ColoredSwitchButtonCell *cell;
-    NSMutableAttributedString *title;
 
     if (entityType == nil) {
         return;
@@ -348,6 +351,7 @@
         [cell setRepresentedObject:entityName];
 #if 0
 #ifndef xxGNUSTEP
+        NSMutableAttributedString *title;
         title = [[[NSMutableAttributedString alloc] initWithString:[entityName description]] autorelease];
 #if defined(__APPLE__)// || defined(GNUSTEP)
         [title replaceCharactersInRange:NSMakeRange(0,0) withString:@"# "];
@@ -415,24 +419,30 @@
 - (NSEnumerator *)enumeratorOfEntitiesTyped:(PajeEntityType *)entityType
                                 inContainer:(PajeContainer *)container
                                    fromTime:(NSDate *)start
-                                     toTime:(NSDate *)end;
+                                     toTime:(NSDate *)end
+                                minDuration:(double)minDuration
 {
     NSEnumerator *origEnum;
     NSMutableSet *filter;
     
     if ([self isHiddenEntityType:entityType]) {
-        NSLog(@"enumerating hidden type %@", entityType);
+        NSWarnLog(@"enumerating hidden type %@", entityType);
         return nil;
     }
 
     origEnum = [inputComponent enumeratorOfEntitiesTyped:entityType
                                              inContainer:container
                                                 fromTime:start
-                                                  toTime:end];
+                                                  toTime:end
+                                             minDuration:minDuration];
 
     filter = [filters objectForKey:entityType];
-    if (filter) {
-        return [[[FilteredEnumerator alloc] initWithEnumerator:origEnum filter:self selector:@selector(isHiddenEntity:filter:) context:filter] autorelease];
+    if (filter != nil) {
+        return [[[FilteredEnumerator alloc] 
+                        initWithEnumerator:origEnum
+                                    filter:self
+                                  selector:@selector(filterHiddenEntity:filter:)
+                                   context:filter] autorelease];
     } else {
         return origEnum;
     }
@@ -443,10 +453,14 @@
     return [hiddenEntityTypes member:entityType] != nil;
 }
 
-- (BOOL)isHiddenEntity:(PajeEntity *)entity
-                filter:(NSSet *)filter
+- (BOOL)filterHiddenEntity:(PajeEntity *)entity
+                    filter:(NSSet *)filter
 {
-    return [filter containsObject:[self nameForEntity:entity]];
+    if ([filter containsObject:[self nameForEntity:entity]]) {
+        return nil;
+    } else {
+        return entity;
+    }
 }
 
 - (NSArray *)allNamesForEntityType:(PajeEntityType *)entityType
@@ -490,7 +504,6 @@
 - (NSDragOperation)matrix:(NSMatrix *)m
           draggingEntered:(id <NSDraggingInfo>)sender
 {
-NSDebugMLog(@"");
     return NSDragOperationAll;
 }
 #endif
@@ -501,7 +514,6 @@ NSDebugMLog(@"");
     NSPoint point = [m convertPoint:[sender draggingLocation] fromView:nil];
     NSButtonCell *cell = [m cellAtPoint:point];
 
-NSDebugMLog(@"");
     [self highlightCell:cell inMatrix:m];
     if (cell)
         return NSDragOperationAll;//NSDragOperationGeneric;
@@ -512,7 +524,6 @@ NSDebugMLog(@"");
 #ifdef GNUSTEP
 - (BOOL)matrix:(NSMatrix *)m prepareForDragOperation:(id <NSDraggingInfo>)sender
 {
-NSDebugMLog(@"");
     return YES;
 }
 #endif
@@ -522,22 +533,19 @@ NSDebugMLog(@"");
     NSPoint point;
     NSButtonCell *cell;
 
-NSDebugMLog(@"");
     point = [m convertPoint:[sender draggingLocation] fromView:nil];
     cell = [m cellAtPoint:point];
 
     [self highlightCell:nil inMatrix:m];
     
     if (cell != nil) {
-        id entityType = [self selectedEntityType];
-NSDebugMLog(@"entityType:%@ color:%@", entityType, [NSColor colorFromPasteboard:[sender draggingPasteboard]]);
-        [inputComponent setColor:[NSColor colorFromPasteboard:[sender draggingPasteboard]]
+        PajeEntityType *entityType = [self selectedEntityType];
+        NSColor *dragColor;
+        
+        dragColor = [NSColor colorFromPasteboard:[sender draggingPasteboard]];
+        [inputComponent setColor:dragColor
                          forName:[cell representedObject]
                     ofEntityType:entityType];
-        if ([cell state] == 1) {
-            [super colorChangedForEntityType:entityType];
-	}
-        [self synchronizeMatrix];
         return YES;
     }
     return NO;
@@ -545,7 +553,6 @@ NSDebugMLog(@"entityType:%@ color:%@", entityType, [NSColor colorFromPasteboard:
 
 - (void)matrix:(NSMatrix *)m draggingExited:(id <NSDraggingInfo>)sender
 {
-NSDebugMLog(@"");
     [self highlightCell:nil inMatrix:m];
 }
 
@@ -573,14 +580,17 @@ NSDebugMLog(@"");
 
 - (void)setConfiguration:(id)config
 {
+    NSDictionary *configuration;
     NSMutableDictionary *hiddenNames;
-    NSArray *hiddenTypes;
+    NSMutableArray *hiddenTypes;
     NSEnumerator *keyEnum;
     NSString *entityTypeName;
     PajeEntityType *entityType;
     int i;
 
-    hiddenNames = [[config objectForKey:@"HiddenNames"] unifyStrings];
+    NSParameterAssert([config isKindOfClass:[NSDictionary class]]);
+    configuration = config;
+    hiddenNames = [[configuration objectForKey:@"HiddenNames"] unifyStrings];
 
     keyEnum = [hiddenNames keyEnumerator];
     while ((entityTypeName = [keyEnum nextObject]) != nil) {
@@ -593,7 +603,8 @@ NSDebugMLog(@"");
         }
     }
 
-    hiddenTypes = [[config objectForKey:@"HiddenEntityTypes"] unifyStrings];
+    hiddenTypes = [[configuration objectForKey:@"HiddenEntityTypes"]
+                                                               unifyStrings];
     for (i = 0; i < [hiddenTypes count];) {
         entityTypeName = [hiddenTypes objectAtIndex:i];
         entityType = [self entityTypeWithName:entityTypeName];
