@@ -24,12 +24,13 @@
 #include <Foundation/Foundation.h>
 #include "../General/PajeFilter.h"
 
-@interface PajeFileReader: PajeComponent
+#define CHUNK_SIZE 1000000
+
+@interface PajeFileReader: PajeComponent <PajeReader>
 {
     NSFileHandle *inputFile;
-    NSString *remainsOfLastRead;
     NSString *inputFilename;
-    unsigned long long bytesScanned;
+    BOOL hasMoreData;
 }
 
 - (id)initWithController:(PajeTraceController *)c;
@@ -37,11 +38,10 @@
 - (void)setInputFilename:(NSString *)filename;
 - (NSString *)inputFilename;
 - (void)readNextChunk;
+- (BOOL)hasMoreData;
 
 - (void)raise:(NSString *)reason;
 
-- (int)eventCount;
-
 - (void)encodeCheckPointWithCoder:(NSCoder *)coder;
 - (void)decodeCheckPointWithCoder:(NSCoder *)coder;
 @end
