@@ -55,7 +55,6 @@
     Assign(endTime, [event objectForKey:@"EndTime"]);
 }
 
-NSDictionary *USER_TYPES;
 - (void)pajeDefineContainerType:(PajeEvent *)event
 {
     NSString *newContainerTypeAlias;
@@ -91,25 +90,27 @@ NSDictionary *USER_TYPES;
         [self error:@"Unknown container type" inEvent:event];
     }
 
-    // new type should not exist
+    // new type should not exist (but may, if replaying)
     if ([userTypes objectForKey:newContainerTypeName] != nil) {
-        NSLog(@"Redefining container type %@", newContainerTypeName);
+        //NSWarnLog(@"Redefining container type %@ with event %@",
+        //          newContainerTypeName, event);
         return;
     }
     if (newContainerTypeAlias != nil 
         && [userTypes objectForKey:newContainerTypeAlias] != nil) {
-        NSLog(@"Redefining container type %@", newContainerTypeAlias);
+        //NSWarnLog(@"Redefining container type alias %@ with event %@",
+        //          newContainerTypeAlias, event);
         return;
     }
 
     // create the new container type
     newContainerType = [PajeContainerType typeWithName:newContainerTypeName
-                                         containerType:containerType];
+                                         containerType:containerType
+                                                 event:event];
     [userTypes setObject:newContainerType forKey:newContainerTypeName];
     if (newContainerTypeAlias != nil) {
         [userTypes setObject:newContainerType forKey:newContainerTypeAlias];
     }
-    USER_TYPES=userTypes;
 }
 
 - (void)pajeDefineLinkType:(PajeEvent *)event
@@ -160,20 +161,22 @@ NSDictionary *USER_TYPES;
 
     // new type should not exist
     if ([userTypes objectForKey:newEntityTypeName] != nil) {
-        NSLog(@"Redefining entity type %@", newEntityTypeName);
+        //NSWarnLog(@"Redefining entity type %@ with event %@",
+        //          newEntityTypeName, event);
         return;
     }
     if (newEntityTypeAlias != nil 
         && [userTypes objectForKey:newEntityTypeAlias] != nil) {
-        NSLog(@"Redefining entity type %@", newEntityTypeAlias);
+        //NSWarnLog(@"Redefining entity type alias %@ with event %@",
+        //          newEntityTypeAlias, event);
         return;
     }
 
     newEntityType = [PajeLinkType typeWithName:newEntityTypeName
                                  containerType:containerType
                            sourceContainerType:sourceContainerType
-                             destContainerType:destContainerType];
-    [newEntityType setEvent:event];
+                             destContainerType:destContainerType
+                                         event:event];
     [userTypes setObject:newEntityType forKey:newEntityTypeName];
     if (newEntityTypeAlias != nil) {
         [userTypes setObject:newEntityType forKey:newEntityTypeAlias];
@@ -205,12 +208,14 @@ NSDictionary *USER_TYPES;
 
     // new type should not exist
     if ([userTypes objectForKey:newEntityTypeName] != nil) {
-        NSLog(@"Redefining entity type %@", newEntityTypeName);
+        //NSWarnLog(@"Redefining entity type %@ with event %@",
+        //          newEntityTypeName, event);
         return;
     }
     if (newEntityTypeAlias != nil 
         && [userTypes objectForKey:newEntityTypeAlias] != nil) {
-        NSLog(@"Redefining entity type %@", newEntityTypeAlias);
+        //NSWarnLog(@"Redefining entity type alias %@ with event %@",
+        //          newEntityTypeAlias, event);
         return;
     }
 
@@ -222,22 +227,24 @@ NSDictionary *USER_TYPES;
     switch (drawingType) {
     case PajeEventDrawingType:
         newEntityType = [PajeEventType typeWithName:newEntityTypeName
-                                      containerType:containerType];
+                                      containerType:containerType
+                                              event:event];
         break;
     case PajeStateDrawingType:
         newEntityType = [PajeStateType typeWithName:newEntityTypeName
-                                      containerType:containerType];
+                                      containerType:containerType
+                                              event:event];
         break;
     case PajeVariableDrawingType:
         newEntityType = [PajeVariableType typeWithName:newEntityTypeName
-                                         containerType:containerType];
+                                         containerType:containerType
+                                                 event:event];
         break;
     default:
         [self error:@"Internal simulator error: unknown drawing type"
             inEvent:event];
     }
 
-    [newEntityType setEvent:event];
     [userTypes setObject:newEntityType forKey:newEntityTypeName];
     if (newEntityTypeAlias != nil) {
         [userTypes setObject:newEntityType forKey:newEntityTypeAlias];
@@ -292,7 +299,7 @@ NSDictionary *USER_TYPES;
     }
 
     if ([entityType drawingType] == PajeVariableDrawingType) {
-        NSLog(@"Values of variables cannot be named in event %@", event);
+        NSWarnLog(@"Values of variables cannot be named in event %@", event);
         return;
     }
 
@@ -351,11 +358,13 @@ NSDictionary *USER_TYPES;
         [self error:@"Unknown container type" inEvent:event];
     }
     
-//    if ([self containerOfNumber:newContainerName 
-//                           type:typeOfNewContainer
-//                        inEvent:event] != nil) {
-//        NSLog(@"Redefining container %@ in event %@", newContainerName, event);
-//    }
+    if ([self containerOfNumber:newContainerName 
+                           type:typeOfNewContainer
+                        inEvent:event] != nil) {
+        //NSWarnLog(@"Redefining container %@ in event %@",
+        //          newContainerName, event);
+        return;
+    }
 
     if (newContainerAlias == nil) {
         newContainerAlias      = newContainerName;
@@ -364,7 +373,9 @@ NSDictionary *USER_TYPES;
         && [self containerOfNumber:newContainerAlias 
                               type:typeOfNewContainer
                            inEvent:event] != nil) {
-        NSLog(@"Redefining container %@ in event %@", newContainerAlias, event);
+        //NSWarnLog(@"Redefining container alias %@ in event %@",
+        //          newContainerAlias, event);
+        return;
     }
 
     container = [self containerOfNumber:containerId
@@ -407,7 +418,7 @@ NSDictionary *USER_TYPES;
 
     containerType = [userTypes objectForKey:containerTypeId];
     if (containerType == nil) {
-        NSLog(@"Unknown container type in event %@", event);
+        NSWarnLog(@"Unknown container type in event %@", event);
         //[self error:@"Unknown container type" inEvent:event];
     }
  
@@ -432,7 +443,7 @@ NSDictionary *USER_TYPES;
     NSString *entityTypeId;
     NSString *containerId;
 
-    if (entityType) {
+    if (entityType != NULL) {
         entityTypeId  = [event objectForKey:@"EntityType"];
 	if (entityTypeId == nil) {
             entityTypeId  = [event objectForKey:@"Type"];
@@ -443,7 +454,7 @@ NSDictionary *USER_TYPES;
         }
     }
 
-    if (entityType != nil && container != nil) {
+    if (entityType != NULL && container != NULL) {
         containerId   = [event objectForKey:@"Container"];
         *container = [self containerOfNumber:containerId
                                         type:[*entityType containerType]
@@ -468,21 +479,17 @@ NSDictionary *USER_TYPES;
 - (void)pajeNewEvent:(PajeEvent *)event
 {
     PajeEventType *entityType;
-    PajeContainer *container;
+    SimulContainer *container;
     id entityValue;
-    UserEvent *userEntity;
 
     [self _getEntityType:&entityType
                    value:&entityValue
                container:&container
                fromEvent:event];
 
-    userEntity = [UserEvent eventWithType:entityType
-                                     name:entityValue
-                                container:container
-                                    event:event];
-
-    [self outputEntity:userEntity];
+    [container newEventWithType:entityType
+                          value:entityValue
+                      withEvent:event];
 }
 
 - (void)pajeSetState:(PajeEvent *)event
