@@ -72,7 +72,12 @@
 {
     [reduceEntityTypes release];
     [entityTypesDictionary release];
-    [typesContainingReducedTypes release];    
+    [typesContainingReducedTypes release];
+    [view removeFromSuperview];
+    [groupByPopUp removeAllItems];
+    [entityTypePopUp removeAllItems];
+    //[nameMatrix removeAllItems];
+    [entityNamePopUp removeAllItems];
     [view release];
     [super dealloc];
 }
@@ -238,8 +243,7 @@
     while ((className = [classNameEnum nextObject]) != nil) {
         entityClass = NSClassFromString(className);
         if (!entityClass) {
-            NSLog(@"%@: class named %@ not found!",
-                  NSStringFromClass([self class]), className);
+            NSWarnMLog(@"class named %@ not found!", className);
             continue;
         }
         [reduceModePopUp addItemWithTitle:[entityClass titleForPopUp]];
@@ -313,7 +317,7 @@
     selectedEntityType = [[entityNamePopUp selectedItem] representedObject];
     if (selectedEntityType) {
         newEntityType = [ReduceEntityType typeWithName:newEntityName
-                                         containerType:[self containerTypeForType:selectedEntityType]
+                                         containerType:(PajeContainerType *)[self containerTypeForType:selectedEntityType]
                                              component:self];
         [newEntityType setEntityClass:[selectedEntityType entityClass]];
         [newEntityType setEntityTypeToReduce:[selectedEntityType entityTypeToReduce]];
@@ -516,17 +520,20 @@
                                 inContainer:(PajeContainer *)container
                                    fromTime:(NSDate *)start
                                      toTime:(NSDate *)end
+                                minDuration:(double)minDuration
 {
     if (![entityType isKindOfClass:[ReduceEntityType class]])
         return [inputComponent enumeratorOfEntitiesTyped:entityType
                                              inContainer:container
                                                 fromTime:start
-                                                  toTime:end];
+                                                  toTime:end
+                                             minDuration:minDuration];
 
     return [(ReduceEntityType *)entityType
                         enumeratorOfEntitiesInContainer:container
                                                fromTime:start
-                                                 toTime:end];
+                                                 toTime:end
+                                            minDuration:minDuration];
 }
 
 
