@@ -35,16 +35,24 @@
 {
     shapefunction *path;
     drawfunction *draw;
+    drawfunction *highlight;
     NSColor *color;
     NSColor *lastColor = nil;
     id entity;
     float x, y, w, h;
     BOOL drawNames;
     NSMutableAttributedString *name;
+    NSDictionary *attributes;
+    
+    attributes = [NSDictionary dictionaryWithObjectsAndKeys:
+                            [NSFont systemFontOfSize:8], @"NSFontAttributeName",
+                            nil];
 
     drawNames = [layout drawsName];
     if (drawNames) {
-        name = [[NSMutableAttributedString alloc] init];
+        name = [[NSMutableAttributedString alloc] 
+                initWithString:@""
+                    attributes:attributes];
     }
     
     y = [layout yInContainer:container];
@@ -52,11 +60,51 @@
     h = [layout height];
     path = [[layout shapeFunction] function];
     draw = [drawFunction function];
+    highlight = [[layout highlightFunction] function];
     PSgsave();
 
     while ((entity = [enumerator nextObject]) != nil) {
         x = TIMEtoX([filter timeForEntity:entity]);
 
+        if ([filter isAggregateEntity:entity]) {
+            float x2 = TIMEtoX([filter endTimeForEntity:entity]);
+            int condensedEntitiesCount;
+            unsigned i;
+            unsigned count;
+            float xi = x;
+            float dx;
+            int sup;
+
+            PSgsave();
+            condensedEntitiesCount = [entity condensedEntitiesCount];
+            sup = [layout isSupEvent];
+
+            count = [filter subCountForEntity:entity];
+            for (i = 0; i < count; i++) {
+                [[filter subColorAtIndex:i forEntity:entity] set];
+                dx = (x2-x) * [filter subCountAtIndex:i forEntity:entity] 
+                            / condensedEntitiesCount;
+                NSRectFill(NSMakeRect(xi, y+(sup*(-h+8))+(!sup*(h-10)), dx, 3));
+                xi += dx;
+            }
+            PSmoveto(x, y+2*sup-2*!sup);
+            PSlineto(x, y-(h-7)*sup+(h-7)*!sup);
+            PSlineto(x2, y-(h-7)*sup+(h-7)*!sup);
+            PSlineto(x2, y+2*sup-2*!sup);
+            if ([filter isSelectedEntity:entity]) {
+                [[NSColor yellowColor] set];
+            } else {
+                [[NSColor blackColor] set];
+            }
+            PSstroke();
+
+            [[NSString stringWithFormat:@"%d", condensedEntitiesCount]
+                        drawAtPoint:NSMakePoint(x, y-(h+1)*sup+(h-8)*!sup)
+                     withAttributes:attributes];
+            PSgrestore();
+
+        } else {
+
         color = [filter colorForEntity:entity];
         if (![color isEqual:lastColor]) {
             lastColor = color;
@@ -65,7 +113,11 @@
 
         PSgsave();
         path(x, y, w, h);
-        draw();
+        if ([filter isSelectedEntity:entity]) {
+            highlight();
+        } else {
+            draw();
+        }
         PSgrestore();
         if (drawNames) {
             PSgsave();
@@ -76,30 +128,26 @@
                 h = 12;
             }
             x -= w;
-            w *= 2;
-            NSRectFill(NSMakeRect(x, y, w, h));
-            NSLog(@"br=%f", [[color colorUsingColorSpaceName:NSCalibratedWhiteColorSpace
-                                          device:nil] whiteComponent]);
+            NSRectFill(NSMakeRect(x, y, w * 2, h));
             if ([[color colorUsingColorSpaceName:NSCalibratedWhiteColorSpace
                                           device:nil] whiteComponent] > 0.5) {
                 [[NSColor blackColor] set];
             } else {
                 [[NSColor whiteColor] set];
             }
-            [name drawInRect:NSMakeRect(x, y, w, h)];
+            [name drawInRect:NSMakeRect(x, y, w * 2, h)];
             PSgrestore();
         }
+        }
     }
     PSgrestore();
 }
 
 #define drawBlackRect(x1, x2, r) \
 do { \
-    lastColor = [NSColor darkGrayColor]; \
-    [lastColor set]; \
-    /*NSRectFill(NSMakeRect(x1, NSMinY(r), x2 - x1, NSHeight(r)));*/ \
-    /*NSFrameRect(NSMakeRect(x1, NSMinY(r), x2 - x1, NSHeight(r)));*/ \
-    NSFrameRect(NSMakeRect(x1, NSMinY(r)+NSHeight(r)/4, x2 - x1 + 1, NSHeight(r)/2)); \
+    [[NSColor darkGrayColor] set]; \
+    NSFrameRect(NSMakeRect(x1, NSMinY(r) + NSHeight(r) / 4, \
+                           x2 - x1 + 1, NSHeight(r) / 2)); \
 } while (0)
 
 - (void)drawStatesWithDescriptor:(STStateTypeLayout *)layout
@@ -107,11 +155,11 @@ do { \
                   fromEnumerator:(NSEnumerator *)enumerator
                     drawFunction:(DrawFunction *)drawFunction
 {
-    shapefunction *path;
-    drawfunction *draw;
     float insetBy;
     NSColor *color;
-    NSColor *lastColor = nil;
+    NSColor *lcolor = [NSColor blackColor];
+    NSColor *hcolor = [NSColor yellowColor];;
+    NSBezierPath *bp = [NSBezierPath bezierPath];
     id <PajeState>entity;
     float firstUndrawnX = MAXFLOAT;
     float lastUndrawnX = MAXFLOAT;
@@ -122,16 +170,17 @@ do { \
 
     drawNames = [layout drawsName];
     if (drawNames) {
-        name = [[NSMutableAttributedString alloc] init];
+        name = [[NSMutableAttributedString alloc] 
+                initWithString:@""
+                    attributes:[NSDictionary dictionaryWithObjectsAndKeys:
+                            [NSFont systemFontOfSize:8], @"NSFontAttributeName",
+                            nil]];
     }
 
     y = [layout yInContainer:container];
     h = [layout height];
-    path = [[layout shapeFunction] function];
-    draw = [drawFunction function];
     insetBy = [layout insetAmount];
-    PSgsave();
-    PSsetlinewidth(1.0);
+    [bp setLineWidth:1.0];
 
     while ((entity = [enumerator nextObject]) != nil) {
         int imbricationLevel;
@@ -145,7 +194,6 @@ do { \
         ow = x2 - x;
 #ifdef GNUSTEP
         // Very big rectangles are not drawn in GNUstep 
-        //rect = NSIntersectionRect(rect, cutRect);
         if (x < NSMinX(cutRect)) x = NSMinX(cutRect);
         if (x2 > NSMaxX(cutRect)) x2 = NSMaxX(cutRect);
 #endif
@@ -160,25 +208,9 @@ do { \
         }
         
         if (w < 2) {
-#if 0
-            if ((firstUndrawnX != MAXFLOAT) && (x > lastUndrawnX)) {
-                drawBlackRect(firstUndrawnX, lastUndrawnX, NSMakeRect(x,y,w,undrawnHeight));
-                undrawnHeight = 0;
-                firstUndrawnX = x;
-            }
-            if (firstUndrawnX == MAXFLOAT) {
-                firstUndrawnX = x;
-            }
-            if (lastUndrawnX < x2) {
-                lastUndrawnX = x2;
-            }
-            if (newHeight > undrawnHeight) {
-                undrawnHeight = newHeight;
-            }
-            continue;
-#else
             if ((lastUndrawnX != MAXFLOAT) && (x2 < firstUndrawnX)) {
-                drawBlackRect(firstUndrawnX, lastUndrawnX, NSMakeRect(x,y,w,undrawnHeight));
+                drawBlackRect(firstUndrawnX, lastUndrawnX,
+                              NSMakeRect(x, y, w, undrawnHeight));
                 undrawnHeight = 0;
                 lastUndrawnX = x2;
             }
@@ -192,36 +224,63 @@ do { \
                 undrawnHeight = newHeight;
             }
             continue;
-#endif
         }
 
         if (firstUndrawnX != MAXFLOAT) {
-            drawBlackRect(firstUndrawnX, lastUndrawnX, NSMakeRect(x,y,w,undrawnHeight));
+            drawBlackRect(firstUndrawnX, lastUndrawnX,
+                          NSMakeRect(x, y, w, undrawnHeight));
             firstUndrawnX = lastUndrawnX = MAXFLOAT;
             undrawnHeight = 0;
         }
 
-        color = [filter colorForEntity:entity];
-        if (![color isEqual:lastColor]) {
-            lastColor = color;
+        NSRect rect;
+        
+        rect = NSMakeRect(x, y, w, newHeight);
+        [bp removeAllPoints];
+        [bp appendBezierPathWithRect:rect];
+        if ([filter isAggregateEntity:entity]) {
+            [bp moveToPoint:NSMakePoint(x, y)];
+            [bp lineToPoint:NSMakePoint(x+w, y+newHeight)];
+            unsigned i;
+            unsigned count = [filter subCountForEntity:entity];
+            for (i = 0; i < count; i++) {
+                float dx;
+                [[filter subColorAtIndex:i forEntity:entity] set];
+                dx = [filter subDurationAtIndex:i forEntity:entity]
+                   * pointsPerSecond;
+                rect.size.width = dx;
+                NSRectFill(rect);
+                rect.origin.x += dx;
+            }
+            if (rect.origin.x <= (x + w - 1)) {
+                [bp moveToPoint:NSMakePoint(rect.origin.x, y+newHeight)];
+                [bp lineToPoint:NSMakePoint(x+w, y)];
+            }
+        } else {
+            color = [filter colorForEntity:entity];
             [color set];
+            NSRectFill(rect);
+
+            if (drawNames && ow > smallEntityWidth && newHeight > 8) {
+                [name replaceCharactersInRange:NSMakeRange(0, [name length])
+                                    withString:[filter nameForEntity:entity]];
+                [name drawInRect:NSMakeRect(ox+2, y + newHeight - 10, ow, 10)];
+            }
         }
 
-        PSgsave();
-        path(x, y, w, newHeight);
-        draw();
-        if (drawNames && ow>10) {
-            [name replaceCharactersInRange:NSMakeRange(0, [name length])
-                                withString:[filter nameForEntity:entity]];
-            [name drawInRect:NSMakeRect(ox+3, y, ow, newHeight)];
+        if ([filter isSelectedEntity:entity]) {
+            [hcolor set];
+        } else {
+            [lcolor set];
         }
-	PSgrestore();
+        [bp stroke];
+
     }
 
     if (firstUndrawnX != MAXFLOAT) {
-        drawBlackRect(firstUndrawnX, lastUndrawnX, NSMakeRect(x, y, w, undrawnHeight));
+        drawBlackRect(firstUndrawnX, lastUndrawnX,
+                      NSMakeRect(x, y, w, undrawnHeight));
     }
-    PSgrestore();
 
     if (drawNames) {
         [name release];
@@ -233,81 +292,107 @@ do { \
                   fromEnumerator:(NSEnumerator *)enumerator
                     drawFunction:(DrawFunction *)drawFunction
 {
-    shapefunction *path;
-    drawfunction *draw;
+    NSBezierPath *path;
     NSColor *color;
     id <PajeState>entity;
+    float yMin;
+    float yMax;
+    float yScale;
+    float yOffset;
+    float off3d;
+    float xOld;
+    float yOld;
+    float yvMin;
+    float yvMax;
     BOOL first = YES;
-    float min;
-    float max;
-    float scale;
-    float offset;
-    float oldx2 = -1e6;
     PajeEntityType *entityType;
-    NSColor *lastColor = nil;
     
     entityType = [layout entityType];
-    
-    min = [layout minValue];
-    max = [layout maxValue];
-    if (max <= min) {
-        min = [[filter minValueForEntityType:entityType] floatValue];
-        max = [[filter maxValueForEntityType:entityType] floatValue];
+
+    yMin = [layout minValue];
+    yMax = [layout maxValue];
+    if (yMax <= yMin) {
+        yMin = [[filter minValueForEntityType:entityType] floatValue];
+        yMax = [[filter maxValueForEntityType:entityType] floatValue];
     }
 
-    if (min != max) {
-        scale = -([layout height] - 4) / (max - min);
+    if (yMin != yMax) {
+        yScale = -([layout height] - 4) / (yMax - yMin);
     } else {
-        scale = 1;
+        yScale = 1;
     }
-    offset = [layout yInContainer:container] + 2 - (max * scale);
 
-    path = [[layout shapeFunction] function];
-    draw = [drawFunction function];
-    color = [filter colorForEntityType:entityType];
-    [color set];
+    off3d = [layout threeD] ? -1 : 0;
+    yOffset = [layout yInContainer:container] + 2 - (yMax * yScale) + off3d;
+
+    path = [[NSBezierPath alloc] init];
+    [path setLineJoinStyle: NSBevelLineJoinStyle];
 
     while ((entity = [enumerator nextObject]) != nil) {
-        float x1 = TIMEtoX([filter startTimeForEntity:entity]);
-        float x2 = TIMEtoX([filter endTimeForEntity:entity]);
-        float y = [[filter valueForEntity:entity] doubleValue] * scale + offset;
-        if (first) {
-            first = NO;
-            PSgsave();
-            PSsetlinewidth([layout lineWidth]);
-            PSmoveto(x1, y);
-        }
+        float x;
+        float y;
 
+        x = TIMEtoX([filter startTimeForEntity:entity]) + off3d;
+        y = [[filter valueForEntity:entity] doubleValue] * yScale + yOffset;
+
+        // do not draw if on same column; get max & min y values
+        if (!first && (xOld - x) < 1) {
+            if (y < yvMin) yvMin = y;
+            if (y > yvMax) yvMax = y;
+            continue;
+        }
 #ifdef GNUSTEP
         // Very big lines are not drawn in GNUstep 
-        if (x1 < NSMinX(cutRect)) x1 = NSMinX(cutRect);
-        if (x2 > NSMaxX(cutRect)) x2 = NSMaxX(cutRect);
+        if (x < NSMinX(cutRect)) x = NSMinX(cutRect);
 #endif
-        if (!first && x1 == oldx2) {
-            PSlineto(x1, y);
+        if (first) {
+            [path moveToPoint: NSMakePoint(NSMaxX(cutRect), y)];
+            [path lineToPoint: NSMakePoint(x, y)];
         } else {
-            PSmoveto(x1, y);
-        }
-
-        color = [filter colorForEntity:entity];
-        if (![color isEqual:lastColor]) {
-            if (lastColor != nil) {
-                PSstroke();
-                PSmoveto(x1, y);
+            if (yvMin != yOld && yvMin != y) {
+                [path lineToPoint: NSMakePoint(xOld, yvMin)];
+                yOld = yvMin;
             }
-            lastColor = color;
-            [color set];
+            if (yvMax != yOld) {
+                [path lineToPoint: NSMakePoint(xOld, yvMax)];
+                yOld = yvMax;
+            }
+            if (y != yOld) {
+                [path lineToPoint: NSMakePoint(xOld, y)];
+            }
+            [path lineToPoint: NSMakePoint(x, y)];
         }
-        PSlineto(x2, y);
-        oldx2 = x2;
+        xOld = x;
+        yvMin = yvMax = yOld = y;
+        first = NO;
     }
-    if (!first) {
-        PSstroke();
-        PSgrestore();
+
+    color = [filter colorForEntityType:entityType];
+
+    if ([layout threeD]) {
+        NSAffineTransform *transform = [NSAffineTransform transform];
+        [path setLineWidth:[layout lineWidth] - 2];
+        [[color highlightWithLevel:0.3] set];
+        [path stroke];
+        [transform translateXBy: 2.0 yBy: 2.0];
+        [path transformUsingAffineTransform: transform];
+        [[color shadowWithLevel:0.3] set];
+        [path stroke];
+        [transform translateXBy: -3.0 yBy: -3.0];
+        [path transformUsingAffineTransform: transform];
+        [color set];
+        [path stroke];
+    } else {
+        [path setLineWidth:[layout lineWidth]];
+        [color set];
+        [path stroke];
     }
+
+    [path release];
 }
 
 
+
 - (void)drawLinksWithDescriptor:(STLinkTypeLayout *)layout
                     inContainer:(PajeContainer *)container
                  fromEnumerator:(NSEnumerator *)enumerator
@@ -315,6 +400,7 @@ do { \
 {
     shapefunction *path;
     drawfunction *draw;
+    drawfunction *highlight;
     NSColor *color;
     NSColor *lastColor = nil;
     id <PajeLink>entity;
@@ -322,13 +408,12 @@ do { \
 
     path = [[layout shapeFunction] function];
     draw = [drawFunction function];
+    highlight = [[layout highlightFunction] function];
 
     PSgsave();
     PSsetlinewidth([layout lineWidth]);
 
     while ((entity = [enumerator nextObject]) != nil) {
-        NSRect rect1;
-        NSRect rect2;
         PajeContainer *sourceContainer;
         PajeContainer *destContainer;
 
@@ -358,6 +443,7 @@ do { \
         }
 
         color = [filter colorForEntity:entity];
+        //color = [color colorWithAlphaComponent:0.5];
         if (![color isEqual:lastColor]) {
             lastColor = color;
             [color set];
@@ -374,7 +460,11 @@ do { \
         } else {
             path(x1, y1, x2-x1, y2-y1);
         }
-        draw();
+        if ([filter isSelectedEntity:entity]) {
+            highlight();
+        } else {
+            draw();
+        }
         PSgrestore();
     }
     PSgrestore();
@@ -384,8 +474,8 @@ do { \
                        inContainer:(PajeContainer *)container
                         insideRect:(NSRect)drawRect
 {
-    id drawStartTime;
-    id drawEndTime;
+    NSDate *drawStartTime;
+    NSDate *drawEndTime;
     NSEnumerator *enumerator;
     NSRect rect;
     PajeDrawingType drawingType;
@@ -410,14 +500,15 @@ do { \
     if (drawingType == PajeEventDrawingType) {
         width = [(STEventTypeLayout *)layoutDescriptor width];
     } else {
-        width = 0;
+        width = 1;
     }
     drawStartTime = XtoTIME(NSMinX(drawRect) - width);
     drawEndTime   = XtoTIME(NSMaxX(drawRect) + width);
     enumerator = [filter enumeratorOfEntitiesTyped:[layoutDescriptor entityType]
                                        inContainer:container
                                           fromTime:drawStartTime
-                                            toTime:drawEndTime];
+                                            toTime:drawEndTime
+                                       minDuration:SMALL_ENTITY_DURATION];
 
     switch (drawingType) {
         case PajeEventDrawingType:
@@ -427,9 +518,6 @@ do { \
                               drawFunction:[layoutDescriptor drawFunction]];
             break;
         case PajeStateDrawingType:
-            // States are ordered by endTime, inset states come first and 
-            // should be drawn last -- reverse the enumeration
-            enumerator = [[enumerator allObjects] reverseObjectEnumerator];
             [self drawStatesWithDescriptor:(STStateTypeLayout *)layoutDescriptor
                                inContainer:container
                             fromEnumerator:enumerator
@@ -442,7 +530,8 @@ do { \
                              drawFunction:[layoutDescriptor drawFunction]];
             break;
         case PajeVariableDrawingType:
-            [self drawValuesWithDescriptor:(STVariableTypeLayout *)layoutDescriptor
+            [self drawValuesWithDescriptor:(STVariableTypeLayout *)
+                                                           layoutDescriptor
                                inContainer:container
                             fromEnumerator:enumerator
                               drawFunction:[layoutDescriptor drawFunction]];
