@@ -1,5 +1,5 @@
 /*
-    Copyright (c) 1998, 1999, 2000, 2001, 2003, 2004 Benhur Stein
+    Copyright (c) 1998-2005 Benhur Stein
     
     This file is part of Paj.
 
@@ -40,7 +40,7 @@
 #import "drawline.h"
 #endif
 
-#include "../General/NSUserDefaults+Colors.h"
+#include "../General/NSUserDefaults+Additions.h"
 #include "../General/Macros.h"
 
 #if defined(GNUSTEP) || defined(__APPLE__)
@@ -54,15 +54,13 @@ void PSInit(void){}
 
 - (void)dealloc
 {
-    [filter release];
-    [startTime release];
-    [endTime release];
-    [backgroundColor release];
-    [selectedBackgroundColor release];
+    Assign(startTime, nil);
+    Assign(endTime, nil);
+    Assign(backgroundColor, nil);
+    Assign(selectedBackgroundColor, nil);
     [super dealloc];
 }
 
-
 - (NSColor *)backgroundColor
 {
     return backgroundColor;
@@ -79,6 +77,7 @@ void PSInit(void){}
     [[NSUserDefaults standardUserDefaults]
               setColor:backgroundColor
                 forKey:DefaultsKey(@"BackgroundColor")];
+    [self setNeedsDisplay:YES];
 }
 
 - (void)setSelectedBackgroundColor:(NSColor *)color
@@ -87,11 +86,12 @@ void PSInit(void){}
     [[NSUserDefaults standardUserDefaults]
               setColor:selectedBackgroundColor
                 forKey:DefaultsKey(@"SelectedBackgroundColor")];
+    [self setNeedsDisplay:YES];
 }
 
 - (void)setFilter:(PajeFilter *)newFilter
 {
-    Assign(filter, newFilter);
+    filter = newFilter;
 }
 
 - (PajeFilter *)filter
@@ -108,6 +108,7 @@ void PSInit(void){}
 {
     Assign(startTime, [filter startTime]);
     Assign(endTime, [filter endTime]);
+    NSDebugMLLog(@"tim", @"Hier changed. times=[%@ %@]", startTime, endTime);
 
     if (startTime != nil && endTime != nil) {
         [self adjustSize];
@@ -145,6 +146,8 @@ void PSInit(void){}
     if (pointsPerSecond == 0) {
         pointsPerSecond = 10000;
     }
+    smallEntityWidth = [[NSUserDefaults standardUserDefaults]
+                              integerForKey:DefaultsKey(@"SmallEntityWidth")];
     hasZoomed = NO;
     filter = nil;
     trackingRectTag = 0;
@@ -210,6 +213,7 @@ void PSInit(void){}
         hasZoomed = NO;
         Assign(oldMiddleTime, nil);
     }
+    newVisible.origin.x = (int)newVisible.origin.x;
     return newVisible;
 }
 
@@ -218,11 +222,6 @@ void PSInit(void){}
     [self adjustSize];
 }
 
-- (id)rootInstance
-{
-    return [filter rootInstance];
-}
-
 - (void)setRulerUnit
 {
     NSRulerView *ruler = [[self enclosingScrollView] horizontalRulerView];
@@ -313,12 +312,20 @@ void PSInit(void){}
 }
 #endif
 
+- (void)removeFromSuperview
+{
+    if (trackingRectTag != 0) {
+        [self removeTrackingRect:trackingRectTag];
+    }
+    [super removeFromSuperview];
+}
+
 - (void)adjustSize
 {
     NSRect newBounds;
     NSScrollView *scrollView = [self enclosingScrollView];
     NSRulerView *ruler = [scrollView horizontalRulerView];
-    id rootInstance = [self rootInstance];
+    id rootInstance = [filter rootInstance];
 
     [self setRulerUnit];
 
@@ -327,12 +334,10 @@ void PSInit(void){}
                afterDelay:0.0];
 
     if (rootInstance != nil) {
-        float delta;
-
         newBounds = [[controller rootLayout] rectOfInstance:rootInstance];
 
         newBounds.origin.x = TIMEtoX(startTime);
-        newBounds.size.width = TIMEtoX(endTime) - newBounds.origin.x;
+        newBounds.size.width = TIMEtoX(endTime) - newBounds.origin.x + 3;
         newBounds.size.height += 1;
         [self setBoundsOrigin:newBounds.origin];
         [self setFrameSize:newBounds.size];
@@ -349,7 +354,9 @@ void PSInit(void){}
         [scrollView tile];
         
         // change mouse tracking rect
-        if (trackingRectTag) [self removeTrackingRect:trackingRectTag];
+        if (trackingRectTag != 0) {
+            [self removeTrackingRect:trackingRectTag];
+        }
         trackingRectTag = [self addTrackingRect:[self visibleRect] owner:self
                                        userData:NULL assumeInside:NO];
     }
@@ -357,8 +364,10 @@ void PSInit(void){}
 
 - (void)verifyTimes:sender
 {
+    //FIXME: shouldn't be necessary; encapsulator should be more intelligent
     NSScrollView *scrollView = [self enclosingScrollView];
-    [filter verifyStartTime:startTime
+    [filter verifyStartTime:/*startTime*/XtoTIME(NSMinX([self convertRect:[scrollView frame]
+                                                    fromView:scrollView]))
                     endTime:XtoTIME(NSMaxX([self convertRect:[scrollView frame]
                                                     fromView:scrollView]))];
 }
@@ -400,6 +409,7 @@ void PSInit(void){}
     }
     Assign(startTime, startTimeLimit);
     Assign(endTime, endTimeLimit);
+    NSDebugMLLog(@"tim", @"times=[%@ %@] trace=[%@ %@]", startTime, endTime, traceStartTime, traceEndTime);
 }
 
 - (void)setPointsPerSecond:(double)pps
@@ -600,44 +610,27 @@ void PSInit(void){}
 
         [selectedBackgroundColor set];
         NSRectFill(selectionRect);
-        
-        /*
-        [[NSColor darkGrayColor] set];
-        PSmoveto(NSMinX(selectionRect), NSMinY(selectionRect));
-        PSlineto(NSMinX(selectionRect), NSMaxY(selectionRect));
-        PSmoveto(NSMaxX(selectionRect), NSMinY(selectionRect));
-        PSlineto(NSMaxX(selectionRect), NSMaxY(selectionRect));
-        PSstroke();
-        */
     }
 }
 
-void PSStrokeLine(float x, float y, float w, float h);
-
-int CiTR;
-BOOL dontdraw;
 
 - (void)drawRect:(NSRect)rect
 {
-    NSDate *drawStartTime;
-    NSDate *drawEndTime;
 
     if (startTime == nil) return;
 
-    // draws some more pixels, for the graphs that expand their limits (events)
-    drawStartTime = XtoTIME(NSMinX(rect) - 5);
-    drawEndTime   = XtoTIME(NSMaxX(rect) + 5);
+[self verifyTimes:self];
 
 #ifdef GNUSTEP
     // Big rectangles are not drawn by GNUstep. Cut them with cutRect.
     cutRect = NSInsetRect([self visibleRect], -2.0, -2.0);
 #endif
-    
+
     NS_DURING
 
         // set blackground
         [self drawBackgroundInRect:rect];
-        
+
         PSsetlinewidth(1);
 
         [self drawInstance:[controller rootInstance]
@@ -658,76 +651,111 @@ BOOL dontdraw;
  * ------------------------------------------------------------------------
  */
 
-- (void)changeSelectionWithPoint:(NSPoint)point
+- (BOOL)isPointInSelection:(NSPoint)point
+{
+    NSDate *pointInTime;
+    
+    if (!selectionExists) {
+        return NO;
+    }
+    
+    pointInTime = XtoTIME(point.x);
+    return [pointInTime isLaterThanDate:selectionStartTime]
+           && [pointInTime isEarlierThanDate:selectionEndTime];
+}
+
+- (void)timeSelectionChanged
 {
     NSRect redisplayRect;
+    NSDate *newSelectionStartTime;
+    NSDate *newSelectionEndTime;
+    double ox1, ox2, nx1, nx2;
+
+    newSelectionStartTime = [controller selectionStartTime];
+    newSelectionEndTime = [controller selectionEndTime];
+    ox1 = TIMEtoX(selectionStartTime);
+    ox2 = TIMEtoX(selectionEndTime);
+    nx1 = TIMEtoX(newSelectionStartTime);
+    nx2 = TIMEtoX(newSelectionEndTime);
+    if (newSelectionStartTime == nil || nx1 >= nx2) {
+        if (selectionExists) {
+            [self setNeedsDisplayFromX:ox1 toX:ox2];
+            Assign(selectionStartTime, nil);
+            Assign(selectionEndTime, nil);
+            selectionExists = NO;
+            [zoomToSelectionButton setEnabled:NO];
+        }
+        return;
+    }
+    if (!selectionExists) {
+        Assign(selectionStartTime, newSelectionStartTime);
+        Assign(selectionEndTime, newSelectionEndTime);
+        selectionExists = YES;
+        [zoomToSelectionButton setEnabled:YES];
+        [self setNeedsDisplayFromX:nx1 toX:nx2];
+        return;
+    }
+
+    if (ox2 < nx1 || nx2 < ox1) {
+        [self setNeedsDisplayFromX:ox1 toX:ox2];
+        Assign(selectionStartTime, newSelectionStartTime);
+        Assign(selectionEndTime, newSelectionEndTime);
+        [self setNeedsDisplayFromX:nx1 toX:nx2];
+        return;
+    }
+
+    redisplayRect = [self visibleRect];
+    if (nx1 != ox1) {
+        [self setNeedsDisplayFromX:MIN(nx1, ox1) toX:MAX(nx1, ox1)];
+    }
+    if (nx2 != ox2) {
+        [self setNeedsDisplayFromX:MIN(nx2, ox2) toX:MAX(nx2, ox2)];
+    }
+
+    Assign(selectionStartTime, newSelectionStartTime);
+    Assign(selectionEndTime, newSelectionEndTime);
+}
+
+- (void)changeSelectionWithPoint:(NSPoint)point
+{
     NSDate *cursorTime;
     NSRect frameRect = [self frame];
 
-//NSLog(@"sel-last:%@ pt=%@", NSStringFromPoint(selectionLastPoint), //NSStringFromPoint(point));
-//NSLog(@"frame=%@", NSStringFromRect(frameRect));
     if (point.x < NSMinX(frameRect)) point.x = NSMinX(frameRect);
     if (point.x > NSMaxX(frameRect)) point.x = NSMaxX(frameRect);
-//NSLog(@"pt=%@", NSStringFromPoint(point));
-
-    if (point.x == selectionLastPoint.x)
-        return;
 
     cursorTime = XtoTIME(point.x);
-    //[cursorTimeField setDoubleValue:[cursorTime timeIntervalSinceDate:startTime] * timeUnitDivisor];
-    [cursorTimeField setDoubleValue:[cursorTime timeIntervalSinceReferenceDate] * timeUnitDivisor];
+    [self setCursorTime:cursorTime];
 
-    redisplayRect = [self visibleRect];
-    if (point.x < selectionLastPoint.x) {
-        redisplayRect.origin.x = point.x;
-        redisplayRect.size.width = selectionLastPoint.x - redisplayRect.origin.x + 1;
+    if ([cursorTime isEarlierThanDate:selectionAnchorTime]) {
+        [controller setSelectionStartTime:cursorTime
+                                  endTime:selectionAnchorTime];
     } else {
-        redisplayRect.origin.x = selectionLastPoint.x - 2;
-        redisplayRect.size.width = point.x - redisplayRect.origin.x + 1;
+        [controller setSelectionStartTime:selectionAnchorTime
+                                  endTime:cursorTime];
     }
 
-    [selectionStartTime release];
-    [selectionEndTime release];
-    if (point.x < selectionAnchorPoint.x) {
-        selectionStartTime = [cursorTime retain];
-        selectionEndTime = [XtoTIME(selectionAnchorPoint.x) retain];
-    } else {
-        selectionEndTime = [cursorTime retain];
-        selectionStartTime = [XtoTIME(selectionAnchorPoint.x) retain];
-    }
-
-    selectionLastPoint = point;
-
-    [self setNeedsDisplayInRect:redisplayRect];
-    [self scrollRectToVisible:redisplayRect];
+    // scroll point to visible
+    NSRect visibleRect;
+    visibleRect = [self visibleRect];
+    visibleRect.origin.x = point.x;
+    visibleRect.size.width = 1;
+    [self scrollRectToVisible:visibleRect];
 }
 
-- (void)setNeedsDisplayInSelection
+- (void)setNeedsDisplayFromX:(double)x1 toX:(double)x2
 {
     NSRect redisplayRect = [self visibleRect];
-    redisplayRect.origin.x = TIMEtoX(selectionStartTime);
-    redisplayRect.size.width = TIMEtoX(selectionEndTime) - redisplayRect.origin.x + 1;
+    redisplayRect.origin.x = x1;
+    redisplayRect.size.width = x2 - x1;
+    redisplayRect = NSIntegralRect(redisplayRect);
     [self setNeedsDisplayInRect:redisplayRect];
 }
 
 - (void)selectAll:(id)sender
 {
-    if (selectionExists) {
-        [selectionStartTime release];
-        [selectionEndTime release];
-    }
-    selectionStartTime = [startTime retain];
-    selectionEndTime = [endTime retain];
-    selectionExists = YES;
-/*    [[NSNotificationCenter defaultCenter]
-              postNotificationName:@"PajeTimeSelectionChangedNotification"
-                            object:[self rootInstance]
-                          userInfo:[NSDictionary dictionaryWithObjectsAndKeys:
-                              selectionStartTime, @"StartTime",
-                              selectionEndTime, @"EndTime", nil]];*/
-    [controller setSelectionStartTime:selectionStartTime
-			      endTime:selectionEndTime];
-    [self setNeedsDisplay:YES];
+    [controller setSelectionStartTime:startTime
+                              endTime:endTime];
 }
 
 
@@ -753,7 +781,7 @@ return NSDragOperationCopy;
 }
 
 #ifdef GNUSTEP
-// this shouldn't be needed (should be the defaul behaviour of NSView)
+// this shouldn't be needed (should be the default behaviour of NSView)
 - (BOOL) prepareForDragOperation: (id <NSDraggingInfo>)sender
 {
     return YES;
@@ -762,24 +790,29 @@ return NSDragOperationCopy;
 - (BOOL)performDragOperation:(id <NSDraggingInfo>)sender
 {
     NSColor *draggedColor;
+    NSPoint point = [self convertPoint:[sender draggingLocation] fromView:nil];
 
     draggedColor = [NSColor colorFromPasteboard:[sender draggingPasteboard]];
-    if (draggedColor == nil)
+    if (draggedColor == nil) {
         return NO;
+    }
 
-    if (highlightedEntity) {
+    // if some entity is highlighted, the color has been dropped over it.
+    if (highlightedEntity != nil) {
         [filter setColor:draggedColor forEntity:highlightedEntity];
         [self setHighlightedEntity:nil];
     } else {
-        //FIXME: should test if inside selection to setSelectedBackgroundcolor:
-        [self setBackgroundColor:draggedColor];
-   }
-    [self setNeedsDisplay:YES];
+        // didn't drop on an entity, change background color
+        if ([self isPointInSelection:point]) {
+            [self setSelectedBackgroundColor:draggedColor];
+        } else {
+            [self setBackgroundColor:draggedColor];
+        }
+    }
     return YES;
 }
 
 
-
 /*
  * Printing
  * ------------------------------------------------------------------------
@@ -793,13 +826,4 @@ return NSDragOperationCopy;
     PSInit();
     [super endPrologue];
 }
-
-
-#ifdef GNUSTEP
-- (void)setCursorTimeField:(id)field { cursorTimeField = [field retain]; }
-- (void)setEntityNameField:(id)field { entityNameField = [field retain]; }
-//- (void)setSelStateButton:(id)button { selStateButton = button; }
-//- (void)setSelEventButton:(id)button { selEventButton = button; }
-//- (void)setSelCommButton:(id)button { selCommButton = button; }
-#endif
 @end
