@@ -382,7 +382,8 @@ if (x != maxLevel/*[thicknesses count]*/) {
         id instance;
 
 #ifdef SHOW_TYPES
-        if (r.size.width > 2 && [layout drawingType] != PajeLinkDrawingType) {
+        if (r.size.width > 2 && [layout drawingType] != PajeLinkDrawingType
+                             && [layout drawingType] != PajeVariableDrawingType) {
             [vcell setStringValue:[layout description]];
             r.origin.x = [self positionForLevel:level] - 1;
             if ([layout isContainer]) {
@@ -391,7 +392,10 @@ if (x != maxLevel/*[thicknesses count]*/) {
 [self widthForLevel:level];
                 r.size.width = NSMaxX([self bounds]) - r.origin.x - 1;
             }
-            [vcell drawWithFrame:r inView:self];
+            [vcell drawWithFrame:NSInsetRect(r, -.5, -.5) inView:self];
+        }
+        if (r.size.width > 2 && [layout drawingType] == PajeVariableDrawingType) {
+            [self widthForLevel:level];
         }
         if (![layout isContainer]) return;
         level++;
@@ -411,14 +415,15 @@ if (x != maxLevel/*[thicknesses count]*/) {
                 if (r.size.width > 2) {
                     if (containerBeingDragged != nil 
                         && [instance isEqual:containerBeingDragged]) {
-                        NSDrawWhiteBezel(r, r);
+                        NSDrawWhiteBezel(NSInsetRect(r, -.5, -.5),
+                                         NSInsetRect(r, -.5, -.5));
                     } else if ([[controller selectedContainers]
                                                      containsObject:instance]) {
                         [[NSColor whiteColor] set];
                         NSRectFill(r);
                     }
                     [cell setStringValue:[instance description]];
-                    [cell drawWithFrame:r inView:self];
+                    [cell drawWithFrame:NSInsetRect(r, -.5, -.5) inView:self];
                 }
             }
 
@@ -440,7 +445,6 @@ if (x != maxLevel/*[thicknesses count]*/) {
     NSRect r;
     float position = [self positionForLevel:level];
     float width = [self widthForLevel:level];
-    DrawView *drawView = (DrawView *)[self clientView];
     
     if (point.x < position) {
         return nil;
@@ -449,7 +453,7 @@ if (x != maxLevel/*[thicknesses count]*/) {
     layoutDescriptor = [controller descriptorForEntityType:containerType];
 
     if (![layoutDescriptor isContainer]) {
-        return;
+        return nil;
     }
 
     r = [layoutDescriptor rectInContainer:container];
@@ -494,7 +498,6 @@ if (x != maxLevel/*[thicknesses count]*/) {
     NSEnumerator *subtypeEnum;
     PajeContainerType *subtype;
     id foundInstance = nil;
-    DrawView *drawView = (DrawView *)[self clientView];
 
     subtypes = [controller containedTypesForContainerType:containerType];
     subtypeEnum = [subtypes objectEnumerator];
@@ -664,10 +667,11 @@ if (x != maxLevel/*[thicknesses count]*/) {
 
     limitRect = [layoutDescriptor rectInContainer:container];
     instanceRect = [(STContainerTypeLayout *)layoutDescriptor rectOfInstance:clickedContainer];
-    //instanceRect.size.height += 1;
+    instanceRect.size.height += 1;
+    instanceRect.size.width += 1;
     
     minY = NSMinY(limitRect);
-    maxY = NSMaxY(limitRect) - NSHeight(instanceRect) /*+ 1*/;
+    maxY = NSMaxY(limitRect) - NSHeight(instanceRect) + 1;
 
     indexBeingDragged = [self levelForPosition:mouseDownLocation.x];
     if (indexBeingDragged == -1) {
@@ -677,7 +681,7 @@ if (x != maxLevel/*[thicknesses count]*/) {
     width = [self widthForLevel:indexBeingDragged];
 
     limitRect.origin.x = instanceRect.origin.x = minX - 1;
-    limitRect.size.width = instanceRect.size.width = width /*+ 1*/;
+    limitRect.size.width = instanceRect.size.width = width + 1;
 
     [self _makeRectVisible:instanceRect];
 
@@ -802,8 +806,6 @@ if (x != maxLevel/*[thicknesses count]*/) {
     NSPoint mouseDownLocation;
     hierarchyRulerPart hitPart;
     
-    NSRect rect = [self bounds];
-    
     mouseDownLocation = [self convertPoint:[event locationInWindow]
                                   fromView:nil];
     
