@@ -1,5 +1,5 @@
 /*
-    Copyright (c) 1998, 1999, 2000, 2001, 2003, 2004 Benhur Stein
+    Copyright (c) 1998-2005 Benhur Stein
     
     This file is part of Paj.
 
@@ -18,7 +18,7 @@
     59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 */
 #include "STEntityTypeLayout.h"
-#include "../General/NSUserDefaults+Colors.h"
+#include "../General/NSUserDefaults+Additions.h"
 #include "../General/Macros.h"
 #include "STController.h"
 
@@ -36,8 +36,9 @@
     case PajeLinkDrawingType:      return [STLinkTypeLayout class];
     case PajeVariableDrawingType:  return [STVariableTypeLayout class];
     case PajeContainerDrawingType: return [STContainerTypeLayout class];
+    default: NSAssert1(0, @"Invalid drawing type %d", dtype);
     }
-    NSAssert1(0, @"Invalid drawing type %d", dtype);
+    return Nil;
 }
 
 + (STEntityTypeLayout *)descriptorWithEntityType:(PajeEntityType *)etype
@@ -79,6 +80,7 @@
                 [self defaultKeyForKey:@" Draw Function"],
             @"PSFillAndFrameWhite",
                 [self defaultKeyForKey:@" Highlight Function"],
+            @"NO", [self defaultKeyForKey:@"DrawsName"],
             nil]];
 }
 
@@ -106,6 +108,19 @@
                 setObject:value forKey:[self defaultKeyForKey:key]];
 }
 
+- (BOOL)defaultBoolForKey:(NSString *)key
+{
+    return [[NSUserDefaults standardUserDefaults] 
+                boolForKey:[self defaultKeyForKey:key]];
+}
+
+- (void)setDefaultBool:(BOOL)value forKey:(NSString *)key
+{
+    return [[NSUserDefaults standardUserDefaults] 
+                setBool:value forKey:[self defaultKeyForKey:key]];
+}
+
+
 - (id)initWithEntityType:(PajeEntityType *)etype
      containerDescriptor:(STContainerTypeLayout *)cDesc
               controller:(STController *)controller
@@ -121,6 +136,8 @@
         [self registerDefaultsWithController:controller];
 
         height = [self defaultFloatForKey:@"Height"];
+        
+        drawsName = [self defaultBoolForKey:@"DrawsName"];
 
         name = [self defaultStringForKey:@" Path Function"];
         Assign(shapeFunction, [ShapeFunction shapeFunctionWithName:name]);
@@ -217,6 +234,7 @@
 - (void)setDrawsName:(BOOL)draws
 {
     drawsName = draws;
+    [self setDefaultBool:draws forKey:@"DrawsName"];
 }
 
 - (BOOL)drawsName
@@ -557,6 +575,7 @@
     [[NSUserDefaults standardUserDefaults] registerDefaults:
         [NSDictionary dictionaryWithObjectsAndKeys:
             tLineWidth, [self defaultKeyForKey:@"LineWidth"],
+            @"NO", [self defaultKeyForKey:@"ThreeD"],
             @"PSNoShape", [self defaultKeyForKey:@" Path Function"],
             @"PSstroke",
                 [self defaultKeyForKey:@" Draw Function"],
@@ -574,6 +593,7 @@
                           controller:controller];
     if (self != nil) {
         lineWidth = [self defaultFloatForKey:@"LineWidth"];
+        threeD = [self defaultBoolForKey:@"ThreeD"];
     }
     return self;
 }
@@ -595,6 +615,16 @@
     return lineWidth;
 }
 
+- (void)setThreeD:(BOOL)flag
+{
+    threeD = flag;
+    [self setDefaultBool:threeD forKey:@"ThreeD"];
+}
+
+- (BOOL)threeD
+{
+    return threeD;
+}
 
 - (void)setMinValue:(float)val
 {
@@ -616,6 +646,16 @@
     return maxValue;
 }
 
+- (float)height
+{
+    return [containerDescriptor heightForVariables];
+}
+
+- (void)setHeight:(float)val
+{
+    [containerDescriptor setHeightForVariables:val];
+}
+
 @end
 
 
@@ -629,6 +669,7 @@
         [NSDictionary dictionaryWithObjectsAndKeys:
             @"1", [self defaultKeyForKey:@"SiblingSeparation"],
             @"1", [self defaultKeyForKey:@"TypeSeparation"],
+            @"60", [self defaultKeyForKey:@"HeightForVariables"],
             nil]];
 }
 
@@ -653,6 +694,7 @@
         
         siblingSeparation = [self defaultFloatForKey:@"SiblingSeparation"];
         subtypeSeparation = [self defaultFloatForKey:@"SubtypeSeparation"];
+        heightForVariables = [self defaultFloatForKey:@"HeightForVariables"];
     }
     return self;
 }
@@ -706,6 +748,17 @@
 }
 
 
+- (void)setHeightForVariables:(float)val
+{
+    heightForVariables = val;
+    [self setDefaultFloat:heightForVariables forKey:@"HeightForVariables"];
+}
+
+- (float)heightForVariables
+{
+    return heightForVariables;
+}
+
 - (void)setSupEventsOffset:(float)val;
 {
     supEventsOffset = val;
@@ -743,6 +796,7 @@
     return (supEventsOffset + infEventsOffset) / 2;
 }
 
+
 - (void)reset
 {
     [super reset];
@@ -921,7 +975,7 @@
             subcontainersOffset += subtypeSeparation;
         }
         [self setOffset:subcontainersOffset ofSubtypes:variableSubtypes];
-        subcontainersOffset += [self getMaxHeight:variableSubtypes];
+        subcontainersOffset += [self heightForVariables];
     }
 
     if (([containerSubtypes count] != 0) && (subcontainersOffset != 0)) {
